<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output encoding="utf-8"/>

<xsl:variable name="service">
    services/katrin.php?target=file&amp;
</xsl:variable>

<xsl:template name="image">
    <xsl:param name="width" />
    <xsl:param name="height" />

    <xsl:variable name="link">
        <xsl:text>setups/katrin/services/adei.fcgi?target=file&amp;</xsl:text>
        <xsl:if test="@run">
            <xsl:text>kdb_run=</xsl:text>
            <xsl:value-of select="@run" />
            <xsl:text>&amp;</xsl:text>
        </xsl:if>
        <xsl:text>kdb_name=</xsl:text>
        <xsl:value-of select="@name" />
    </xsl:variable>

    <xsl:element name="a">
        <xsl:attribute name="href">
            <xsl:value-of select="$link" />
            <xsl:text>.png</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="target">_blank</xsl:attribute>
        <xsl:element name="img">
            <xsl:attribute name="src">
                <xsl:value-of select="$link" />
                <xsl:text>_pv.png</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="border">0</xsl:attribute>
            <xsl:attribute name="alt">
                <xsl:value-of select="." />
            </xsl:attribute>
            <xsl:attribute name="width">
                <xsl:value-of select="$width" />
            </xsl:attribute>
            <xsl:attribute name="height">
                <xsl:value-of select="$height" />
            </xsl:attribute>
        </xsl:element>
    </xsl:element>

</xsl:template>

<xsl:template match="image" mode="small">
    <xsl:call-template name="image">
        <xsl:with-param name="width" select="'150'" />
        <xsl:with-param name="height" select="'100'" />
    </xsl:call-template>
</xsl:template>

<xsl:template match="image" mode="normal">
    <xsl:call-template name="image">
        <xsl:with-param name="width" select="'300'" />
        <xsl:with-param name="height" select="'200'" />
    </xsl:call-template>
</xsl:template>

<xsl:template match="image" mode="large">
    <xsl:call-template name="image">
        <xsl:with-param name="width" select="'600'" />
        <xsl:with-param name="height" select="'400'" />
    </xsl:call-template>
</xsl:template>

<xsl:template match="image" mode="table">
    <td>
        <xsl:apply-templates select="." mode="small" />
    </td>
</xsl:template>

<xsl:template match="image" mode="title">
    <th><xsl:value-of select="." /></th>
</xsl:template>

<xsl:template match="file">
    <p style="margin: 0">
        <xsl:element name="a">
            <xsl:attribute name="href">
                <xsl:text>setups/katrin/services/adei.fcgi?target=file</xsl:text>
                <xsl:text>&amp;kdb_download=yes</xsl:text>
                <xsl:if test="@run">
                    <xsl:text>&amp;kdb_run=</xsl:text>
                    <xsl:value-of select="@run" />
                </xsl:if>
                <xsl:if test="@name">
                    <xsl:text>&amp;kdb_name=</xsl:text>
                    <xsl:value-of select="@name" />
                </xsl:if>
            </xsl:attribute>
            <xsl:value-of select="." />
        </xsl:element>
        <xsl:if test="@filesize">
            <span style="font-size: small">
                <xsl:text> (</xsl:text>
                <xsl:value-of select="@filesize" />
                <xsl:text>)</xsl:text>
            </span>
        </xsl:if>
    </p>
</xsl:template>

<!-- ADEI links and images -->
<xsl:template name="adeiurl">
    <xsl:text>db_server=</xsl:text>
    <xsl:value-of select="@server" />
    <xsl:text>&amp;db_name=</xsl:text>
    <xsl:value-of select="@database" />
    <xsl:text>&amp;db_group=</xsl:text>
    <xsl:value-of select="@group" />
    <xsl:if test="@mask">
        <xsl:text>&amp;db_mask=</xsl:text>
        <xsl:value-of select="@mask" />
    </xsl:if>
    <xsl:if test="@window">
        <xsl:text>&amp;window=</xsl:text>
        <xsl:value-of select="@window" />
    </xsl:if>
</xsl:template>

<xsl:template match="adei">
    <xsl:element name="a">
        <xsl:attribute name="href">
            <xsl:text>javascript:adei.config.Load('</xsl:text>
            <xsl:call-template name="adeiurl" />
            <xsl:text>&amp;module=graph&amp;setup=katrin, true)</xsl:text>
        </xsl:attribute>
        <xsl:value-of select="." />
    </xsl:element>
</xsl:template>

<xsl:template match="plots">
    <table style="margin:10px 0">
        <tr>
            <xsl:apply-templates select="image" mode="title" />
        </tr>
        <tr>
            <xsl:apply-templates select="image" mode="table" />
        </tr>
    </table>
</xsl:template>

<xsl:template match="streams">
    <xsl:for-each select="type">
        <xsl:variable name="title">
            <xsl:choose>
                <xsl:when test="@name=uw">UW</xsl:when>
                <xsl:when test="@name='ipe3_energy'">IPE3 Energy</xsl:when>
                <xsl:when test="@name='ipe3_waveform'">IPE3 Waveform</xsl:when>
                <xsl:when test="@name='ipe3_histogram'">IPE3 Histogram</xsl:when>
                <xsl:when test="@name='ipe4_energy'">IPE4 Energy</xsl:when>
                <xsl:when test="@name='ipe4_waveform'">IPE4 Waveform</xsl:when>
                <xsl:when test="@name='ipe4_histogram'">IPE4 Histogram</xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:element name="img">
            <xsl:attribute name="src">
                <xsl:text>setups/katrin/images/</xsl:text>
                <xsl:value-of select="@name" />
                <xsl:text>.png</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="height">16</xsl:attribute>
            <xsl:attribute name="alt">
                <xsl:value-of select="$title" />
            </xsl:attribute>
            <xsl:attribute name="title">
                <xsl:value-of select="$title" />
            </xsl:attribute>
            <xsl:attribute name="style">
                <xsl:text>margin: 0 5px; vertical-align: middle</xsl:text>
            </xsl:attribute>
        </xsl:element>
    </xsl:for-each>
</xsl:template>

<xsl:template match="downloads">
    <h4>Downloads</h4>
    <xsl:apply-templates select="file" />
</xsl:template>

<xsl:template match="transmission">
    <div>
        <h4>
	    <xsl:text>Transmission (</xsl:text>
	    <xsl:value-of select="@parameter" />
	    <xsl:text>):</xsl:text>
        </h4>
        <xsl:apply-templates select="image" mode="large" />
    </div>
</xsl:template>

<xsl:template match="parameter">
    <div>
        <h4>
	    <xsl:text>Control parameter (</xsl:text>
	    <xsl:value-of select="@parameter" />
	    <xsl:text>):</xsl:text>
	</h4>
        <xsl:apply-templates select="image" mode="normal" />
    </div>
</xsl:template>

<xsl:template match="countrate">
    <div>
        <h4>Eventrate:</h4>
        <xsl:apply-templates select="image" mode="normal" />
    </div>
</xsl:template>

<xsl:template match="errors">
    <tr>
        <td colspan="2">
            <p class="errors">
                <xsl:for-each select="error">
                    <span class="service">
                        <xsl:value-of select="@service" />
                    </span>
                    <xsl:text>: </xsl:text>
                    <xsl:value-of select="." />
                    <br />
                </xsl:for-each>
            </p>
        </td>
    </tr>
</xsl:template>

<xsl:template match="parameters">
    <xsl:if test="fluke">
        <tr>
            <td><b>Voltage:</b></td>
            <td><xsl:value-of select="fluke" /> V</td>
        </tr>
    </xsl:if>
</xsl:template>

<xsl:template match="run">
    <table>
        <tr>
            <td>
                <h2>Run <xsl:value-of select="identifier" /></h2>
            </td>
            <td>
                <a href="javascript:katrin.SetCustomProperties('kdb_run=')">
                    Back to run overview
                </a>
            </td>
        </tr>
        <tr>
            <td colspan="2" style="padding: 0 10px">
                <xsl:apply-templates select="streams" />
            </td>
        </tr>
        <xsl:apply-templates select="errors" />
        <tr>
            <td rowspan="1" style="width:600px; padding: 0 10px 0 0">
                <xsl:apply-templates select="plots/transmission" />
                <table style="font-size:small">
                    <tr>
                        <td><b>Start:</b></td>
                        <td><xsl:value-of select="start" /></td>
                    </tr>
                    <tr>
                        <td><b>Duration:</b></td>
                        <td><xsl:value-of select="duration" /></td>
                    </tr>
                    <tr>
                        <td><b>Configuration:</b></td>
                        <td><xsl:value-of select="configuration" /></td>
                    </tr>
                    <tr>
                        <td><b>ORCA version:</b></td>
                        <td><xsl:value-of select="orca_version" /></td>
                    </tr>
                    <xsl:apply-templates select="parameters" />
                </table>
            </td>
            <td>
                <xsl:apply-templates select="plots/countrate" />
                <xsl:apply-templates select="plots/parameter" />
                <xsl:apply-templates select="downloads" />
            </td>
        </tr>
        <tr>
            <td>
                <xsl:apply-templates select="plots" />
            </td>
            <td style="vertical-align:bottom">
                <div>
                    <xsl:if test="../subruns">
                        <a href="javascript:katrin.SetCustomProperties('kdb_subruns=0')">
                            Hide subruns
                        </a>
                    </xsl:if>
                    <xsl:if test="not(../subruns)">
                        <a href="javascript:katrin.SetCustomProperties('kdb_subruns=1')">
                            Show subruns
                        </a>
                    </xsl:if>
                </div>
            </td>
        </tr>
    </table>
</xsl:template>

<xsl:template match="subrun">
    <div style="margin: 20px 0">
        <h4>Subrun <xsl:value-of select="identifier" /></h4>
        <table style="font-size:small">
            <tr>
                <td><b>Start:</b></td>
                <td><xsl:value-of select="start" /></td>
            </tr>
            <tr>
                <td><b>End:</b></td>
                <td><xsl:value-of select="end" /></td>
            </tr>
            <xsl:apply-templates select="parameters" />
        </table>
        <xsl:apply-templates select="plots" />
    </div>
</xsl:template>

<xsl:template match="subruns">
    <div>
        <xsl:apply-templates select="subrun" />
    </div>
</xsl:template>

<xsl:template match="/result">
    <div style="margin: 10px">
        <xsl:choose>
            <xsl:when test="Error">
                <h2>Error</h2>
                <xsl:value-of select="Error" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="run" />
                <xsl:apply-templates select="subruns" />
            </xsl:otherwise>
        </xsl:choose>
    </div>
</xsl:template>
</xsl:stylesheet>
